import React, { useEffect } from 'react';
import { useImmer } from 'use-immer';
import { Box, Newline, Text } from 'ink';
import Spinner from 'ink-spinner';
import { v4 as uuid } from 'uuid';

import { MigrationParams } from '../../index';

interface Props {
	migration: (params: MigrationParams) => void;
}

const MigrationProgress: React.FC<Props> = ({ migration }) => {
	const [steps, setSteps] = useImmer<
		({ key: string; text: string } & (
			| {
					state: 'in_progress' | 'success';
					error?: Error | undefined;
			  }
			| { state: 'fail'; error: Error }
		))[]
	>([]);

	useEffect(() => {
		const addStep = (text: string) => {
			setSteps((curSteps) => {
				curSteps.push({ key: uuid(), text, state: 'in_progress' });
			});
		};

		const completeStep = () => {
			setSteps((curSteps) => {
				curSteps[curSteps.length - 1].state = 'success';
			});
		};

		const failStep = (e: Error) => {
			setSteps((curSteps) => {
				if (!curSteps.length) {
					console.error(e);
					return;
				}
				curSteps[curSteps.length - 1].state = 'fail';
				curSteps[curSteps.length - 1].error = e;
			});
		};

		const done = () => {};

		(async () => {
			await migration({
				addStep,
				completeStep,
				done,
			});
		})().catch((e) => {
			failStep(e);
		});
	}, [migration]);

	return (
		<>
			{steps.map((step) => (
				<Box key={step.key} flexDirection='row'>
					<Box marginRight={1}>
						{step.state === 'in_progress' ? (
							<Text>
								[<Spinner type='dots' />]
							</Text>
						) : step.state === 'success' ? (
							<Text>
								[<Text color='green'>✓</Text>]
							</Text>
						) : (
							<Text>
								[<Text color='red'>×</Text>]
							</Text>
						)}
					</Box>

					{step.state === 'fail' ? (
						<Text>
							{step.text}
							<Newline />
							<Text color='red'>{step.error.message}</Text>
						</Text>
					) : (
						<>
							<Text>{step.text}</Text>
						</>
					)}
				</Box>
			))}
		</>
	);
};

export default MigrationProgress;
