import React, {
  useCallback,
  useEffect,
  useState,
} from 'react';
import { Box, Text } from 'ink';
import TextInput from 'ink-text-input';
import SelectInput from 'ink-select-input';

interface Props<T extends string = string> {
  text: React.ReactFragment;
  defaultValue?: T;
  selectFrom?: { value: T; label: string }[];
  onDone: (value: T) => void;
}

const Prompt = <T extends string = string>({
  text,
  defaultValue,
  selectFrom,
  onDone,
}: React.PropsWithChildren<Props<T>>) => {
  const [promptValue, setValue] = useState('' as T);
  const [isDone, setDone] = useState(false);

  const onSelect = useCallback(
    (val: T) => {
      setValue(val);
      setDone(true);
    },
    [setValue, onDone],
  );

  const onSubmit = useCallback(() => {
    if (!promptValue && defaultValue) {
      setValue(defaultValue);
    }
    setDone(true);
  }, [setDone, onDone, promptValue, defaultValue]);

  useEffect(() => {
    if (isDone) {
      onDone(promptValue);
    }
  }, [isDone, promptValue]);

  const prefix = (
    <Box flexDirection="row">
      <Box marginRight={1}>
        {isDone ? (
          <Text>
            [
            <Text color="green">✓</Text>
            ]
          </Text>
        ) : (
          <Text>
            [
            <Text color="gray">?</Text>
            ]
          </Text>
        )}
      </Box>
      <Text>
        {text}
        {defaultValue && ` <${defaultValue}>`}
        :
        {' '}
      </Text>
    </Box>
  );

  return (
    <>
      {selectFrom ? (
        <Box flexDirection={isDone ? 'row' : 'column'}>
          {prefix}
          {isDone ? (
            <Text>{promptValue}</Text>
          ) : (
            <SelectInput
              items={selectFrom}
              onSelect={({ value }) => onSelect(value)}
              initialIndex={0}
            />
          )}
        </Box>
      ) : (
        <Box flexDirection="row">
          {prefix}
          {isDone ? (
            <Text>{promptValue}</Text>
          ) : (
            <TextInput
              value={promptValue}
              onChange={(value) => setValue(value as T)}
              onSubmit={onSubmit}
            />
          )}
        </Box>
      )}
    </>
  );
};

export default Prompt;
